/*
    This file is part of web3.js.
    web3.js is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    web3.js is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License
    along with web3.js.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * @file index.js
 * @author Samuel Furter <samuel@ethereum.org>
 * @date 2018
 */

import {PromiEvent} from 'web3-core-promievent';
import {ProvidersModuleFactory} from 'web3-providers';
import * as Utils from 'web3-utils';
import {formatters} from 'web3-core-helpers';
import {MethodModuleFactory} from 'web3-core-method';
import {ContractModuleFactory} from 'web3-eth-contract';
import {AbiCoder} from 'web3-eth-abi';
import {Network} from 'web3-net';
import EnsModuleFactory from './factories/EnsModuleFactory';

/**
 * Returns the Ens object
 *
 * @method Ens
 *
 * @param {HttpProvider|WebsocketProvider|IpcProvider|EthereumProvider|String} provider
 * @param {Accounts} accounts
 * @param {Object} options
 *
 * @returns {Ens}
 */
export const Ens = (provider, accounts, options) => {
    const abiCoder = new AbiCoder();

    const methodModuleFactory = new MethodModuleFactory();

    return new EnsModuleFactory().createENS(
        provider,
        new ProvidersModuleFactory(),
        new MethodModuleFactory(accounts),
        new ContractModuleFactory(Utils, formatters, abiCoder, accounts, methodModuleFactory),
        PromiEvent,
        abiCoder,
        Utils,
        formatters,
        new Network(provider),
        {},
        options
    );
};
